#!/bin/bash

MOLE_HOME=`cd \`dirname $0\` > /dev/null 2>&1 ; echo $PWD`

mkdir -p cycles-proc/

cd cycles

for p in `ls` ; do
  if ps x | grep idiom-search-one | grep -v grep | \
    sed 's/.*-one.sh //' | grep "^$p$" ; then
    echo "Search in package $p still running"
    continue
  fi

  # when running classify concurrently, only run one process for each package
  if ! mkdir ../cycles-proc/$p > /dev/null 2>&1; then
    continue
  fi

  # normalise cycles and identify invalid ones, split critical vs. uniproc
  echo $p
  grep -r -B1 ^Cycle: $p/ | \
    perl -n -e '
      chomp;
      if(/^.*-Source: (.*?)\s*$/) { print "$1;"; }
      elsif(/^.*:Cycle: (.*?)\s*$/) { print "$1\n"; }' > \
    ../cycles-proc/$p/all-cycles.txt
  cat ../cycles-proc/$p/all-cycles.txt | \
    $MOLE_HOME/filter-critical-cycles.pl > \
    ../cycles-proc/$p/filtered-cycles.txt
  grep ';critical$' ../cycles-proc/$p/filtered-cycles.txt > \
    ../cycles-proc/$p/critical.txt
  grep ';uniproc$' ../cycles-proc/$p/filtered-cycles.txt > \
    ../cycles-proc/$p/uniproc.txt
done
cd ..

cd cycles-proc
for p in * ; do
  if ! ( set -o noclobber; echo > $p/critical-classified.txt ) 2> /dev/null ; then
    continue
  fi

  echo "Classify $p [`cat $p/critical.txt | wc -l`]"

  cat $p/uniproc.txt | $MOLE_HOME/classify-uniproc.pl > $p/uniproc-classified.txt

  cat $p/critical.txt | $MOLE_HOME/classify-critical.pl > $p/critical-classified.txt

  touch $p/classify-done
done

cd ..

