#!/usr/bin/perl -w

use strict;

sub is_uniproc()
{
  my @cycle=split(/\s/, $_[0]);
  my $ret=1;

  scalar(@cycle) or die "Empty cycle\n";

  my $front=shift @cycle;
  $ret=($front =~ /^(Pos..|DpData..|Rf.|Fr.|Ws.)$/) ? 1 : 0;

  if($ret)
  {
    $ret=&is_uniproc(join(" ", @cycle)) if(scalar(@cycle));
  }

  return $ret;
}

sub is_thinair()
{
  my @cycle=split(/\s/, $_[0]);
  my $ret=1;

  scalar(@cycle) or die "Empty cycle\n";

  my $front=shift @cycle;
  $ret=($front =~ /^(Po...|DpData..|Rfe)$/) ? 1 : 0;

  if($ret)
  {
    $ret=&is_thinair(join(" ", @cycle)) if(scalar(@cycle));
  }

  return $ret;
}

sub consume_prop()
{
  my @cycle=split(/\s/, $_[0]);

  while(scalar(@cycle))
  {
    my $front=shift @cycle;

    next if($front =~ /^(Po...|DpData..|Rf.|Fr.)$/);

    unshift @cycle, $front;
    last;
  }

  return join(" ", @cycle);
}

sub is_observation()
{
  if(!($_[0]=~/Fre/))
  {
    return 0;
  }

  my @cycle=split(/\s/, $_[0]);

  scalar(@cycle) or die "Empty cycle\n";

  my $front=shift @cycle;

  if($front ne "Fre")
  {
    push @cycle, $front;
    return &is_observation(join(" ", @cycle));
  }

  $front=shift @cycle;
  if(&consume_prop($front) ne "")
  {
    return 0;
  }

  return scalar(@cycle) ? &is_thinair(join(" ", @cycle)) : 1;
}

sub is_propagation()
{
  my @cycle=split(/\s/, $_[0]);
  my $ret=1;

  scalar(@cycle) or die "Empty cycle\n";

  while(scalar(@cycle) && $ret)
  {
    @cycle=split(/\s/, &consume_prop(join(" ", @cycle)));

    if(scalar(@cycle))
    {
      my $front=shift @cycle;
      $ret=($front =~ /^(Ws.)$/) ? 1 : 0;
    }
  }

  return $ret;
}

while(<>)
{
  chomp;
  my $line=$_;
  /^(.+);(.+);(.+);(.+);(critical|uniproc);(.+)$/ or die "Invalid input $_\n";

  my $cycle=$4;
  $cycle =~ s/(Lw)?Sync/Po/g;

  if(&is_uniproc($cycle))
  {
    print "$line;uniproc\n";
  }
  elsif(&is_thinair($cycle))
  {
    print "$line;thinair\n";
  }
  elsif(&is_observation($cycle))
  {
    print "$line;observation\n";
  }
  elsif(&is_propagation($cycle))
  {
    print "$line;propagation\n";
  }
  else
  {
    print "$line;unknown\n";
  }
}

