#!/bin/bash

MOLE_HOME=`cd \`dirname $0\` > /dev/null 2>&1 ; echo $PWD`

export PKG=$1

if [ ! -f cycles-proc/$PKG/mole-report.txt ] ; then
  echo "mole not done"
  exit 1
fi

export URLNAME=""
BASEURL="`zgrep "http://sources.debian.net" \
  cycles-proc/$PKG/mole-report.txt cycles-proc/$PKG/mole-source-links-*.txt.gz | \
  head -n 1 | sed 's#^.*http://##' | sed 's# .*$##'`"
BASEURL="`echo $BASEURL | sed 's/^sources\.debian\.net\/src\/\([^\/]\+\/[^\/]\+\)\/.*$/http:\/\/sources.debian.net\/src\/\1/'`"
if [ -n "$BASEURL" ] ; then
  export URLNAME="`echo $PKG | tr '0-9\.\-' 'ABCDEFGHIJDH'`"
fi

cat > cycles-proc/$PKG/mole-report.tex <<EOF
\\section*{$PKG}

The results listed below were generated using mole based on these \\ahref{http://dkr-debian.cs.ox.ac.uk:8080/job/goto-cc-$PKG/ws/goto-binaries.tar.bz2}{goto binaries}.
EOF

if [ -n "$URLNAME" ] ; then
  echo "\\urldef{\\$URLNAME}{\\url}{$BASEURL}" >> cycles-proc/$PKG/mole-report.tex
fi

cat >> cycles-proc/$PKG/mole-report.tex <<EOF

\\begin{tabular}{|l|r|l|l|l|l|l|}
\\hline
Pattern & Occurrences & All cycles & Cycle & Objects & Source locations & Source events \\\\ \\hline
EOF

$MOLE_HOME/mole-latex-transform.pl < cycles-proc/$PKG/mole-report.txt >> cycles-proc/$PKG/mole-report.tex

cat >> cycles-proc/$PKG/mole-report.tex <<EOF
\\end{tabular}
EOF

for f in cycles-proc/$PKG/mole-source-links-*.txt.gz ; do
  if [ ! -f $f ] ; then
    break
  fi

  s=`basename $f .txt.gz`
  s=`echo $s | sed 's/mole-source-links-//'`
  o="cycles-proc/$PKG/mole-report-$s"

  cat > $o.tex <<EOF
\\documentclass{article}
\\usepackage{url}
\\usepackage{hevea}

\\begin{document}

\\section*{All instances of $s in $PKG}
EOF

  if [ -n "$URLNAME" ] ; then
    echo "\\urldef{\\$URLNAME}{\\url}{$BASEURL}" >> $o.tex
  fi

  cat >> $o.tex <<EOF
\\begin{tabular}{|l|l|l|l|}
\\hline
Cycle & Objects & Source locations & Source events \\\\ \\hline
EOF

  zcat $f | $MOLE_HOME/mole-latex-transform.pl >> $o.tex

  cat >> $o.tex <<EOF
\\end{tabular}
\\end{document}
EOF
done

